
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.19.1',
    'branch_nick': 'patch-0-1_19_1_preparation',
    'revision_id': 'bc3df4643243ec47d5a5cca3f54b215c6c3f892d',
    'revno': 7801
}
